@@ -1,4 +1,9 @@
 /*
+ * The copyright in this software is being made available under the 2-clauses 
+ * BSD License, included below. This software may be subject to other third 
+ * party and contributor rights, including patent rights, and no such rights
+ * are granted under this license.
+ *
  * Copyright (c) 2001-2003, David Janssens
  * Copyright (c) 2002-2003, Yannick Verschueren
  * Copyright (c) 2003-2005, Francois Devaux and Antonin Descampe
@@ -58,7 +63,7 @@
 
 /** @name Local static functions */
 /*@{*/
-unsigned int ops;
+/*unsigned int ops;*/
 /**
 Forward lazy transform (horizontal)
 */
@@ -668,7 +673,7 @@ void dwt_encode(opj_tcd_tilecomp_t * tilec, int dwtid[3]) {
 	int *bj = NULL;
 	int *cj = NULL;
 	
-	ops = 0;
+	/*ops = 0;*/
 
 	memset(flagnorm,0,8000*sizeof(int));
 	w = tilec->x1-tilec->x0;
